﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using C1.WPF.FlexGrid;
using C1.WPF;
using System.ComponentModel;
using System.Data;

namespace C1TreeViewC1FlexGrid_DragDrop
{
    public partial class MainWindow : Window
    {
        MyCellFactory _cellFactory = null;
        C1DragDropManager _dragDropMng;
        public ObservableCollection<Item> _droppedItems = new ObservableCollection<Item>();
        IDictionary<int, SubCategory> _subCategoryDirectory = null;
        GroupRow gr;
        public MainWindow()
        {
            InitializeComponent();

            #region "Grid settings"
            SampleGrid.EnableFiltering(true);
            SampleGrid.AllowDragging = AllowDragging.Columns;
            SampleGrid.AllowResizing = AllowResizing.None;
            SampleGrid.Columns["Delete"].AllowDragging = false;
            SampleGrid.SelectionMode = C1.WPF.FlexGrid.SelectionMode.Row;
            SampleGrid.ItemsSource = _droppedItems;                             //bind grid to the items dropped from treeview
            SampleGrid.AutoSizeColumns(0, SampleGrid.Columns.Count, 4);         //resizes all the grid columns according to their content
            SampleGrid.RowHeaders.Columns[0].Visible = false;                   //hide the fixed column
            #endregion

            #region "Grid cell factory"
            _cellFactory = new MyCellFactory(SampleGrid);
            SampleGrid.CellFactory = _cellFactory; //tell grid to use our custom CellFactory
            #endregion

            #region "Group Row"
            gr = new GroupRow(); //create GroupRow to show total at grid bottom
            gr.FontWeight = FontWeights.Bold;
            gr.FontSize = 14;
            gr.Height = 30;
            gr.Background = new SolidColorBrush(Color.FromArgb(0xff, 0x8f, 0xb4, 0xcc));
            gr.Foreground = new SolidColorBrush(Colors.White);
            gr.HorizontalAlignment = HorizontalAlignment.Center;
            SampleGrid.ColumnFooters.Rows.Add(gr);
            gr.Visible = false;
            SampleGrid.ColumnFooters.Columns["Unit"].Format = "Total units: 0";
            SampleGrid.ColumnFooters.Columns["ItemAmount1"].Format = "Total Bill: $0.00";
            #endregion

            #region "Grid events"
            SampleGrid.MouseLeftButtonDown += SampleGrid_MouseLeftButtonDown;
            #endregion

            #region "Tree settings"
            SampleTreeView.ItemsSource = BuildTreeView();
            SampleTreeView.AllowDragDrop = true;
            SampleTreeView.SelectionMode = C1SelectionMode.Single;
            #endregion

            #region "Tree events"
            SampleTreeView.MouseLeftButtonUp += SampleTreeView_MouseLeftButtonUp;
            SampleTreeView.ItemPrepared += SampleTreeView_ItemPrepared;
            #endregion

            #region "Drag drop manager"
            _dragDropMng = new C1DragDropManager();
            _dragDropMng.RegisterDragSource(SampleTreeView, DragDropEffect.Copy, ModifierKeys.None,true);   //tell dragdrop manager to allow dragging from treeview
            _dragDropMng.RegisterDropTarget(SampleGrid, true);              //tell dragdrop manager to allow dropping over grid
            _dragDropMng.DragStart += Ddm_DragStart;
            _dragDropMng.DragDrop += Ddm_DragDrop;
            #endregion
        }

        private void SampleGrid_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            if (SampleGrid.HitTest(e.GetPosition(SampleGrid)).CellType == CellType.Cell)
            {
                HitTestInfo hti = SampleGrid.HitTest(e.GetPosition(SampleGrid));   // get column and row under cursor
                if (hti.Column == 0 && hti.Row > -1)                               
                {
                    var product = SampleGrid.Rows[hti.Row].DataItem as Item;
                    if (product != null)
                    {
                        _droppedItems.Remove(product);                             // if column clicked is first column then perform row deletion
                        product.Reset();
                        SampleGrid.ItemsSource = _droppedItems;
                        SampleGrid.Select(-1, -1);
                        if(_droppedItems.Count==0)
                        {
                            gr.Visible = false;                                  // hide summary row when there are no dropped items
                        }
                    }
                }
            }
        }
        private void Ddm_DragStart(object source, DragDropEventArgs e)
        {
            if((e.DragSource as C1TreeView) !=null)
            {
                e.Effect = DragDropEffect.None;
            }
            if((e.DragSource as C1TreeViewItem) !=null && (e.DragSource as C1TreeViewItem).HasItems)
            {
                e.Effect = DragDropEffect.None;
            }
        }
        private void Ddm_DragDrop(object source, DragDropEventArgs e)
        {
            List<SubCategory> list = _subCategoryDirectory.ToList().Select(t => t.Value).ToList();
           
            if (e.DropTarget is C1FlexGrid && (e.DragSource as C1TreeViewItem) !=null && (e.DragSource as C1TreeViewItem).DataContext is Item)
            {
                Item item = (e.DragSource as C1TreeViewItem).DataContext as Item;
                item.PropertyChanged += Item_PropertyChanged;
                if (_droppedItems.Contains(item))
                {
                    // if item dropped is already a member of grid, then highlight it and increase its quantity by one
                    foreach (Row r in SampleGrid.Rows)
                    {
                        if (r.DataItem == item)
                        {
                            SampleGrid.SelectedIndex = r.Index;
                            SampleGrid[r.Index, SampleGrid.Columns.IndexOf("Unit")] = Convert.ToInt32(SampleGrid[r.Index, SampleGrid.Columns.IndexOf("Unit")]) + 1;
                        }
                    }
                }
                else
                {
                    SampleGrid.SelectedItems.Clear();
                    _droppedItems.Add(item);                 //add dropped Item to grid data source
                    gr.Visible = true;                       //show the summary row
                }
                SampleGrid.ItemsSource=  _droppedItems;    
                SampleGrid.AutoSizeColumn(SampleGrid.Columns["Name"].Index, 10);
                SampleGrid.Columns["SubCategoryID"].ValueConverter = new ColumnValueConverter(list, "SubCategoryID", "SubCategoryName");        //translates column values to reflect Display value
            }
        }
        private void Item_PropertyChanged(object sender, PropertyChangedEventArgs e)
        {
            SampleGrid.Invalidate();
        }
        private void SampleTreeView_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            try
            {
                if (SampleTreeView.ItemContainerGenerator.Status == System.Windows.Controls.Primitives.GeneratorStatus.ContainersGenerated && SampleTreeView.SelectedItem.DataContext is SubCategory)
                {
                    ExpandNode((SampleTreeView.SelectedItem.DataContext as SubCategory));       //expands the node to show sub-items
                }
            }
            catch (Exception)
            { }
        }
        private void SampleTreeView_ItemPrepared(object sender, ItemPreparedEventArgs e)
        {
            if (e.Item is Item)
            {  // if item being dragged is of type Item then allow
                C1TreeViewItem node = (C1TreeViewItem)SampleTreeView.ItemContainerGenerator.ContainerFromItem(_subCategoryDirectory[(e.Item as Item).SubCategoryID]);
                C1TreeViewItem innernode = (C1TreeViewItem)node.ItemContainerGenerator.ContainerFromItem(e.Item);
                _dragDropMng.RegisterDragSource(innernode, DragDropEffect.Move, ModifierKeys.None, true);
            }
            else if (e.Item is SubCategory)
            {   // if item being dragged is of type SubCategory, then disallow drag
                C1TreeViewItem node = (C1TreeViewItem)SampleTreeView.ItemContainerGenerator.ContainerFromItem(e.Item);
                _dragDropMng.RegisterDragSource(node, DragDropEffect.Move, ModifierKeys.None, true);
            }
        }
        public void ExpandNode(SubCategory subCat)
        {   // if clicked item is node(category) then expand it, to show its Items
            C1TreeViewItem node = (C1TreeViewItem)SampleTreeView.ItemContainerGenerator.ContainerFromItem(subCat);
            if(node.IsExpanded)
            {
                node.Collapse();
            }
            else
            {
                node.Expand();
            }
        }
        private ObservableCollection<object> BuildTreeView()
        {
            ObservableCollection<object> itemsSource = new ObservableCollection<object>();
            IDictionary<int, Category> categoryDirectory = new Dictionary<int, Category>();
            _subCategoryDirectory = new Dictionary<int, SubCategory>();
            List<Category> categories = DataLoader.LoadCategories(); 
            foreach (Category c in categories)
            {
                categoryDirectory.Add(c.CategoryID, c);     // add all categories to a categories collection
            }

            List<SubCategory> subCategories = DataLoader.LoadSubCategories();
            foreach (SubCategory sc in subCategories)
            {
                itemsSource.Add(sc);
                _subCategoryDirectory.Add(sc.SubCategoryID, sc);    // add all sub-categories to a categories collection
                Category c = categoryDirectory[sc.CategoryID];      // get parent Category associated with Sub-Category
                c.subCategories.Add(sc);                            // and add Sub-Category to its Category's SubCategory Collection
            }

            List<Item> items = DataLoader.LoadItems();
            foreach (Item i in items)
            {
                SubCategory sc = _subCategoryDirectory[i.SubCategoryID];        // get parent Sub-Category associated with Item  
                sc.items.Add(i);                                                // and add Item to its SubCategory's Items Collection

            }

            return itemsSource;
        }
    }
    public class MyCellFactory: C1.WPF.FlexGrid.CellFactory
    {
        public override void CreateCellContent(C1FlexGrid grid, Border bdr, CellRange rng)
        {
            var col = grid.Columns[rng.Column];
            var row = grid.Rows[rng.Row];
            if (col.ColumnName == "Brand")
            {
                var binding = new Binding();
                binding.Path = new PropertyPath("BrandName");
                binding.Converter = new MyImageConverter();  //use custom-converter to show Brand Logo matching BrandName

                var img = new Image();
                img.SetBinding(Image.SourceProperty, binding);  
               
                bdr.Child = img;
                bdr.Height = 100;
            }
            else if (col.ColumnName == "Delete")
            {
                var img = new Image();
                Binding binding = new Binding();
                binding.Converter = new RemoveRowConverter();  //use custom converter to show Remove row image in RemoveButton

                img.SetBinding(Image.SourceProperty, binding);
                bdr.Child = null;
                bdr.Child = img;
            }
            else
            {
                base.CreateCellContent(grid, bdr, rng);
            }
        }

        C1FlexGrid _flex;
        public MyCellFactory(C1FlexGrid flex)
        {
            _flex = flex;
        }
    }
    public class MyImageConverter: IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            string path = null;
            path = value.ToString();
            if(path != null)
            {
                var uri = string.Format("pack://application:,,,/Logos/{0}.png", path);  //here path should be BrandName
                return new BitmapImage(new Uri(uri));
            }
            return null;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }
    public class RemoveRowConverter : IValueConverter
    {
        public object Convert(object value, Type targetType, object parameter, CultureInfo culture)
        {
            string path = null;
            path = value.ToString();
            if (path != null)
            {
                var uri = string.Format("pack://application:,,,/Logos/remove.png", path);
                return new BitmapImage(new Uri(uri));
            }
            return null;
        }

        public object ConvertBack(object value, Type targetType, object parameter, CultureInfo culture)
        {
            throw new NotImplementedException();
        }
    }

}
