﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Runtime.CompilerServices;

namespace C1TreeViewC1FlexGrid_DragDrop
{
    public class Item :INotifyPropertyChanged
    {
        public enum NeckTypeEnum { VNeck=0, Round, Collar}
        public enum BrandEnum { Adidas=0, AllenSolly, ColorPlus, JohnPlayers, Lee, Levis, Puma, UCB, VanHeusen}
        public enum SizeEnum { XS, S, M, L, XL }
        public int ItemID { get; set; }
        public string Name { get; set; }
        public string Title { get; set; }
        public int SubCategoryID { get; set; }
        public int CategoryID { get; set; }
        public NeckTypeEnum Neck { get; set; }
        public BrandEnum BrandName { get; set; }
        public double Price { get; set; }

        private int _unit = 1;
        public int Unit
        {
            get
            {
                return _unit;
            }
            set
            {
                if (value != _unit)
                {
                    _unit = value;
                    TotalPrice = value * Price;
                    NotifyPropertyChanged();
                }
            }
        }

        private double _totalPrice = 0;
        public double TotalPrice
        {
            get {
                if (_totalPrice == 0 || Unit != 0 || Price != 0)
                {
                    _totalPrice = _unit * Price;
                }
                return _totalPrice;
            }
            set
            {
                if (_totalPrice != value)
                {
                    _totalPrice = value;
                    NotifyPropertyChanged();
                }
            }
        }
        public Item()
        {
        }
        public void Reset()
        {
            _unit = 1;
            NotifyPropertyChanged();
        }
        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged([CallerMemberName] String propertyName = "")
        {
            if (PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }
        public override bool Equals(object obj)
        {
            Item itm = obj as Item;
            bool isEqual = false;

            if (itm != null)
            {
                isEqual = (itm.ItemID == this.ItemID);
            }
            return isEqual;
        }
        public Item Clone()
        {
            return this.MemberwiseClone() as Item;
        }
    }
}
